@@ -45,6 +45,8 @@
 #include "config.h"
 #include <math.h>
 
+#include <gio/gio.h>
+
 #include "pango-context.h"
 #include "pango-font-private.h"
 #include "pangofc-fontmap-private.h"
@@ -219,6 +221,8 @@ static PangoFontset *pango_fc_font_map_load_fontset  (PangoFontMap
 static void          pango_fc_font_map_list_families (PangoFontMap                 *fontmap,
 						       PangoFontFamily            ***families,
 						       int                          *n_families);
+static PangoFontFamily *pango_fc_font_map_get_family (PangoFontMap *fontmap,
+                                                      const char   *name);
 
 static double pango_fc_font_map_get_resolution (PangoFcFontMap *fcfontmap,
 						PangoContext   *context);
@@ -226,6 +230,11 @@ static PangoFont *pango_fc_font_map_new_font   (PangoFcFontMap    *fontmap,
 						PangoFcFontsetKey *fontset_key,
 						FcPattern         *match);
 
+static PangoFontFace *pango_fc_font_map_get_face (PangoFontMap *fontmap,
+                                                  PangoFont    *font);
+
+static void pango_fc_font_map_changed (PangoFontMap *fontmap);
+
 static guint    pango_fc_font_face_data_hash  (PangoFcFontFaceData *key);
 static gboolean pango_fc_font_face_data_equal (PangoFcFontFaceData *key1,
 					       PangoFcFontFaceData *key2);
@@ -835,8 +844,7 @@ filter_fontset_by_format (FcFontSet *fontset)
       const char *s;
 
       res = FcPatternGetString (fontset->fonts[i], FC_FONTFORMAT, 0, (FcChar8 **)(void*)&s);
-      g_assert (res == FcResultMatch);
-      if (pango_fc_is_supported_font_format (s))
+      if (res == FcResultMatch && pango_fc_is_supported_font_format (s))
         FcFontSetAdd (result, FcPatternDuplicate (fontset->fonts[i]));
     }
 
@@ -1115,8 +1123,49 @@ pango_fc_fontset_foreach (PangoFontset           *fontset,
  * PangoFcFontMap
  */
 
+static GType
+pango_fc_font_map_get_item_type (GListModel *list)
+{
+  return PANGO_TYPE_FONT_FAMILY;
+}
+
+static void ensure_families (PangoFcFontMap *fcfontmap);
+
+static guint
+pango_fc_font_map_get_n_items (GListModel *list)
+{
+  PangoFcFontMap *fcfontmap = PANGO_FC_FONT_MAP (list);
+
+  ensure_families (fcfontmap);
+
+  return fcfontmap->priv->n_families;
+}
+
+static gpointer
+pango_fc_font_map_get_item (GListModel *list,
+                            guint       position)
+{
+  PangoFcFontMap *fcfontmap = PANGO_FC_FONT_MAP (list);
+
+  ensure_families (fcfontmap);
+
+  if (position < fcfontmap->priv->n_families)
+    return g_object_ref (fcfontmap->priv->families[position]);
+
+  return NULL;
+}
+
+static void
+pango_fc_font_map_list_model_init (GListModelInterface *iface)
+{
+  iface->get_item_type = pango_fc_font_map_get_item_type;
+  iface->get_n_items = pango_fc_font_map_get_n_items;
+  iface->get_item = pango_fc_font_map_get_item;
+}
+
 G_DEFINE_ABSTRACT_TYPE_WITH_CODE (PangoFcFontMap, pango_fc_font_map, PANGO_TYPE_FONT_MAP,
-                                  G_ADD_PRIVATE (PangoFcFontMap))
+                                  G_ADD_PRIVATE (PangoFcFontMap)
+                                  G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, pango_fc_font_map_list_model_init))
 
 static void
 pango_fc_font_map_init (PangoFcFontMap *fcfontmap)
@@ -1191,7 +1240,10 @@ pango_fc_font_map_class_init (PangoFcFontMapClass *class)
   fontmap_class->load_font = pango_fc_font_map_load_font;
   fontmap_class->load_fontset = pango_fc_font_map_load_fontset;
   fontmap_class->list_families = pango_fc_font_map_list_families;
+  fontmap_class->get_family = pango_fc_font_map_get_family;
+  fontmap_class->get_face = pango_fc_font_map_get_face;
   fontmap_class->shape_engine_type = PANGO_RENDER_TYPE_FC;
+  fontmap_class->changed = pango_fc_font_map_changed;
 }
 
 
@@ -1357,26 +1409,13 @@ is_alias_family (const char *family_name)
 }
 
 static void
-pango_fc_font_map_list_families (PangoFontMap      *fontmap,
-				 PangoFontFamily ***families,
-				 int               *n_families)
+ensure_families (PangoFcFontMap *fcfontmap)
 {
-  PangoFcFontMap *fcfontmap = PANGO_FC_FONT_MAP (fontmap);
   PangoFcFontMapPrivate *priv = fcfontmap->priv;
   FcFontSet *fontset;
   int i;
   int count;
 
-  if (priv->closed)
-    {
-      if (families)
-	*families = NULL;
-      if (n_families)
-	*n_families = 0;
-
-      return;
-    }
-
   if (priv->n_families < 0)
     {
       FcObjectSet *os = FcObjectSetBuild (FC_FAMILY, FC_SPACING, FC_STYLE, FC_WEIGHT, FC_WIDTH, FC_SLANT,
@@ -1406,8 +1445,7 @@ pango_fc_font_map_list_families (PangoFontMap      *fontmap,
 	  PangoFcFamily *temp_family;
 
 	  res = FcPatternGetString (fontset->fonts[i], FC_FONTFORMAT, 0, (FcChar8 **)(void*)&s);
-	  g_assert (res == FcResultMatch);
-          if (!pango_fc_is_supported_font_format (s))
+          if (res != FcResultMatch || !pango_fc_is_supported_font_format (s))
             continue;
 
 	  res = FcPatternGetString (fontset->fonts[i], FC_FAMILY, 0, (FcChar8 **)(void*)&s);
@@ -1450,6 +1488,27 @@ pango_fc_font_map_list_families (PangoFontMap      *fontmap,
 
       priv->n_families = count;
     }
+}
+
+static void
+pango_fc_font_map_list_families (PangoFontMap      *fontmap,
+				 PangoFontFamily ***families,
+				 int               *n_families)
+{
+  PangoFcFontMap *fcfontmap = PANGO_FC_FONT_MAP (fontmap);
+  PangoFcFontMapPrivate *priv = fcfontmap->priv;
+
+  if (priv->closed)
+    {
+      if (families)
+	*families = NULL;
+      if (n_families)
+	*n_families = 0;
+
+      return;
+    }
+
+  ensure_families (fcfontmap);
 
   if (n_families)
     *n_families = priv->n_families;
@@ -1458,6 +1517,29 @@ pango_fc_font_map_list_families (PangoFontMap      *fontmap,
     *families = g_memdup (priv->families, priv->n_families * sizeof (PangoFontFamily *));
 }
 
+static PangoFontFamily *
+pango_fc_font_map_get_family (PangoFontMap *fontmap,
+                              const char   *name)
+{
+  PangoFcFontMap *fcfontmap = PANGO_FC_FONT_MAP (fontmap);
+  PangoFcFontMapPrivate *priv = fcfontmap->priv;
+  int i;
+
+  if (priv->closed)
+    return NULL;
+
+  ensure_families (fcfontmap);
+
+  for (i = 0; i < priv->n_families; i++)
+    {
+      PangoFontFamily *family = PANGO_FONT_FAMILY (priv->families[i]);
+      if (strcmp (name, pango_font_family_get_name (family)) == 0)
+        return family;
+    }
+
+  return NULL;
+}
+
 static double
 pango_fc_convert_weight_to_fc (PangoWeight pango_weight)
 {
@@ -1677,6 +1759,26 @@ pango_fc_font_map_new_font (PangoFcFontMap    *fcfontmap,
   return (PangoFont *)fcfont;
 }
 
+static PangoFontFace *
+pango_fc_font_map_get_face (PangoFontMap *fontmap,
+                            PangoFont    *font)
+{
+  PangoFcFont *fcfont = PANGO_FC_FONT (font);
+  FcResult res;
+  const char *s;
+  PangoFontFamily *family;
+
+  res = FcPatternGetString (fcfont->font_pattern, FC_FAMILY, 0, (FcChar8 **) &s);
+  g_assert (res == FcResultMatch);
+
+  family = pango_font_map_get_family (fontmap, s);
+
+  res = FcPatternGetString (fcfont->font_pattern, FC_STYLE, 0, (FcChar8 **)(void*)&s);
+  g_assert (res == FcResultMatch);
+
+  return pango_font_family_get_face (family, s);
+}
+
 static void
 pango_fc_default_substitute (PangoFcFontMap    *fontmap,
 			     PangoFcFontsetKey *fontsetkey,
@@ -1876,15 +1978,31 @@ pango_fc_font_map_load_fontset (PangoFontMap                 *fontmap,
 void
 pango_fc_font_map_cache_clear (PangoFcFontMap *fcfontmap)
 {
+  guint removed, added;
+
   if (G_UNLIKELY (fcfontmap->priv->closed))
     return;
 
+  removed = fcfontmap->priv->n_families;
+
   pango_fc_font_map_fini (fcfontmap);
   pango_fc_font_map_init (fcfontmap);
+   
+  ensure_families (fcfontmap);
+
+  added = fcfontmap->priv->n_families;
+
+  g_list_model_items_changed (G_LIST_MODEL (fcfontmap), 0, removed, added);
 
   pango_font_map_changed (PANGO_FONT_MAP (fcfontmap));
 }
 
+static void
+pango_fc_font_map_changed (PangoFontMap *fontmap)
+{
+  /* we emit GListModel::changed in pango_fc_font_map_cache_clear() */
+}
+
 /**
  * pango_fc_font_map_config_changed:
  * @fcfontmap: a #PangoFcFontMap
@@ -1903,9 +2021,9 @@ pango_fc_font_map_config_changed (PangoFcFontMap *fcfontmap)
 }
 
 /**
- * pango_fc_font_map_set_config:
+ * pango_fc_font_map_set_config: (skip)
  * @fcfontmap: a #PangoFcFontMap
- * @fcconfig: (nullable): a #FcConfig, or %NULL
+ * @fcconfig: (nullable): a `FcConfig`, or %NULL
  *
  * Set the FcConfig for this font map to use.  The default value
  * is %NULL, which causes Fontconfig to use its global "current config".
@@ -1947,12 +2065,14 @@ pango_fc_font_map_set_config (PangoFcFontMap *fcfontmap,
 }
 
 /**
- * pango_fc_font_map_get_config:
+ * pango_fc_font_map_get_config: (skip)
  * @fcfontmap: a #PangoFcFontMap
  *
- * Fetches FcConfig attached to a font map.  See pango_fc_font_map_set_config().
+ * Fetches the `FcConfig` attached to a font map.
  *
- * Returns: (nullable): the #FcConfig object attached to @fcfontmap, which
+ * See also: pango_fc_font_map_set_config()
+ *
+ * Returns: (nullable): the `FcConfig` object attached to @fcfontmap, which
  *          might be %NULL.
  *
  * Since: 1.38
@@ -2129,7 +2249,7 @@ _pango_fc_font_map_fc_to_coverage (FcCharSet *charset)
  * needed for correct operation on the #PangoContext after calling
  * this function.
  *
- * Return value: a new #PangoContext
+ * Return value: (transfer full): a new #PangoContext
  *
  * Since: 1.4
  *
@@ -2495,6 +2615,14 @@ pango_fc_face_is_synthesized (PangoFontFace *face)
   return fcface->fake;
 }
 
+static PangoFontFamily *
+pango_fc_face_get_family (PangoFontFace *face)
+{
+  PangoFcFace *fcface = PANGO_FC_FACE (face);
+
+  return PANGO_FONT_FAMILY (fcface->family);
+}
+
 static void
 pango_fc_face_finalize (GObject *object)
 {
@@ -2522,6 +2650,7 @@ pango_fc_face_class_init (PangoFcFaceClass *class)
   class->get_face_name = pango_fc_face_get_face_name;
   class->list_sizes = pango_fc_face_list_sizes;
   class->is_synthesized = pango_fc_face_is_synthesized;
+  class->get_family = pango_fc_face_get_family;
 }
 
 
@@ -2531,7 +2660,48 @@ pango_fc_face_class_init (PangoFcFaceClass *class)
 
 typedef PangoFontFamilyClass PangoFcFamilyClass;
 
-G_DEFINE_TYPE (PangoFcFamily, pango_fc_family, PANGO_TYPE_FONT_FAMILY)
+static GType
+pango_fc_family_get_item_type (GListModel *list)
+{
+  return PANGO_TYPE_FONT_FACE;
+}
+
+static void ensure_faces (PangoFcFamily *family);
+
+static guint
+pango_fc_family_get_n_items (GListModel *list)
+{
+  PangoFcFamily *fcfamily = PANGO_FC_FAMILY (list);
+
+  ensure_faces (fcfamily);
+
+  return (guint)fcfamily->n_faces;
+}
+
+static gpointer
+pango_fc_family_get_item (GListModel *list,
+                          guint       position)
+{
+  PangoFcFamily *fcfamily = PANGO_FC_FAMILY (list);
+
+  ensure_faces (fcfamily);
+
+  if (position < fcfamily->n_faces)
+    return g_object_ref (fcfamily->faces[position]);
+
+  return NULL;
+}
+
+static void
+pango_fc_family_list_model_init (GListModelInterface *iface)
+{
+  iface->get_item_type = pango_fc_family_get_item_type;
+  iface->get_n_items = pango_fc_family_get_n_items;
+  iface->get_item = pango_fc_family_get_item;
+}
+
+G_DEFINE_TYPE_WITH_CODE (PangoFcFamily, pango_fc_family, PANGO_TYPE_FONT_FAMILY,
+                         G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, pango_fc_family_list_model_init))
 
 static PangoFcFace *
 create_face (PangoFcFamily *fcfamily,
@@ -2551,20 +2721,10 @@ create_face (PangoFcFamily *fcfamily,
 }
 
 static void
-pango_fc_family_list_faces (PangoFontFamily  *family,
-			    PangoFontFace  ***faces,
-			    int              *n_faces)
+ensure_faces (PangoFcFamily *fcfamily)
 {
-  PangoFcFamily *fcfamily = PANGO_FC_FAMILY (family);
   PangoFcFontMap *fcfontmap = fcfamily->fontmap;
-  PangoFcFontMapPrivate *priv;
-
-  *faces = NULL;
-  *n_faces = 0;
-  if (G_UNLIKELY (!fcfontmap))
-    return;
-
-  priv = fcfontmap->priv;
+  PangoFcFontMapPrivate *priv = fcfontmap->priv;
 
   if (fcfamily->n_faces < 0)
     {
@@ -2673,6 +2833,19 @@ pango_fc_family_list_faces (PangoFontFamily  *family,
 	  fcfamily->faces = faces;
 	}
     }
+}
+
+static void
+pango_fc_family_list_faces (PangoFontFamily  *family,
+			    PangoFontFace  ***faces,
+			    int              *n_faces)
+{
+  PangoFcFamily *fcfamily = PANGO_FC_FAMILY (family);
+
+  if (G_UNLIKELY (!fcfamily->fontmap))
+    return;
+
+  ensure_faces (fcfamily);
 
   if (n_faces)
     *n_faces = fcfamily->n_faces;
@@ -2681,6 +2854,29 @@ pango_fc_family_list_faces (PangoFontFamily  *family,
     *faces = g_memdup (fcfamily->faces, fcfamily->n_faces * sizeof (PangoFontFace *));
 }
 
+static PangoFontFace *
+pango_fc_family_get_face (PangoFontFamily *family,
+                          const char      *name)
+{
+  PangoFcFamily *fcfamily = PANGO_FC_FAMILY (family);
+  int i;
+
+  ensure_faces (fcfamily);
+
+  if (name == NULL)
+    name = "Regular"; /* This name always exists in fontconfig */
+
+  for (i = 0; i < fcfamily->n_faces; i++)
+    {
+      PangoFontFace *face = PANGO_FONT_FACE (fcfamily->faces[i]);
+
+      if (strcmp (name, pango_font_face_get_face_name (face)) == 0)
+        return face;
+    }
+
+  return NULL;
+}
+
 static const char *
 pango_fc_family_get_name (PangoFontFamily  *family)
 {
@@ -2734,6 +2930,7 @@ pango_fc_family_class_init (PangoFcFamilyClass *class)
   object_class->finalize = pango_fc_family_finalize;
 
   class->list_faces = pango_fc_family_list_faces;
+  class->get_face = pango_fc_family_get_face;
   class->get_name = pango_fc_family_get_name;
   class->is_monospace = pango_fc_family_is_monospace;
   class->is_variable = pango_fc_family_is_variable;
@@ -2745,6 +2942,17 @@ pango_fc_family_init (PangoFcFamily *fcfamily)
   fcfamily->n_faces = -1;
 }
 
+/**
+ * pango_fc_font_map_get_hb_face: (skip)
+ * @fcfontmap: a #PangoFcFontMap
+ * @fcfont: a #PangoFcFont
+ *
+ * Retrieves the `hb_face_t` for the given #PangoFcFont.
+ *
+ * Returns: (transfer none) (nullable): the `hb_face_t` for the given Pango font
+ *
+ * Since: 1.44
+ */
 hb_face_t *
 pango_fc_font_map_get_hb_face (PangoFcFontMap *fcfontmap,
                                PangoFcFont    *fcfont)
@@ -2752,11 +2960,14 @@ pango_fc_font_map_get_hb_face (PangoFcFontMap *fcfontmap,
   PangoFcFontFaceData *data;
 
   data = pango_fc_font_map_get_font_face_data (fcfontmap, fcfont->font_pattern);
-  
+
   if (!data->hb_face)
     {
       hb_blob_t *blob;
 
+      if (!hb_version_atleast (2, 0, 0))
+        g_error ("Harfbuzz version too old (%s)\n", hb_version_string ());
+
       blob = hb_blob_create_from_file (data->filename);
       data->hb_face = hb_face_create (blob, data->id);
       hb_blob_destroy (blob);
