@@ -53,7 +53,10 @@ test_attributes_basic (void)
   test_copy (pango_attr_font_desc_new (desc));
   pango_font_description_free (desc);
   test_copy (pango_attr_underline_new (PANGO_UNDERLINE_LOW));
+  test_copy (pango_attr_underline_new (PANGO_UNDERLINE_ERROR_LINE));
   test_copy (pango_attr_underline_color_new (100, 200, 300));
+  test_copy (pango_attr_overline_new (PANGO_OVERLINE_SINGLE));
+  test_copy (pango_attr_overline_color_new (100, 200, 300));
   test_copy (pango_attr_strikethrough_new (TRUE));
   test_copy (pango_attr_strikethrough_color_new (100, 200, 300));
   test_copy (pango_attr_rise_new (256));
@@ -63,6 +66,9 @@ test_attributes_basic (void)
   test_copy (pango_attr_shape_new (&rect, &rect));
   test_copy (pango_attr_gravity_new (PANGO_GRAVITY_SOUTH));
   test_copy (pango_attr_gravity_hint_new (PANGO_GRAVITY_HINT_STRONG));
+  test_copy (pango_attr_allow_breaks_new (FALSE));
+  test_copy (pango_attr_show_new (PANGO_SHOW_SPACES));
+  test_copy (pango_attr_insert_hyphens_new (FALSE));
 }
 
 static void
@@ -94,7 +100,7 @@ assert_attributes (GSList     *attrs,
   s = g_string_new ("");
   print_attributes (attrs, s);
   g_assert_cmpstr (s->str, ==, expected);
-  g_string_free (s, FALSE);
+  g_string_free (s, TRUE);
 }
 
 static void
@@ -352,6 +358,15 @@ test_iter (void)
   PangoAttrIterator *copy;
   gint start, end;
 
+  /* Empty list */
+  list = pango_attr_list_new ();
+  iter = pango_attr_list_get_iterator (list);
+
+  g_assert_false (pango_attr_iterator_next (iter));
+  g_assert_null (pango_attr_iterator_get_attrs (iter));
+  pango_attr_iterator_destroy (iter);
+  pango_attr_list_unref (list);
+
   list = pango_attr_list_new ();
   attr = pango_attr_size_new (10);
   pango_attr_list_insert (list, attr);
